package org.eclipse.hyades.logging.commons;

import org.apache.commons.logging.Log;
import org.eclipse.hyades.internal.logging.core.Constants;
import org.eclipse.hyades.logging.core.IExternalizableToXml;
import org.eclipse.hyades.logging.core.IPreemptedLogger;
import org.eclipse.hyades.logging.core.LoggingAgent;
import org.eclipse.hyades.logging.events.cbe.CommonBaseEvent;
import org.eclipse.hyades.logging.events.cbe.CompletionException;
import org.eclipse.hyades.logging.events.cbe.EventFactory;
import org.eclipse.hyades.logging.events.cbe.EventFactoryHome;
import org.eclipse.hyades.logging.events.cbe.impl.EventFactoryContext;
import org.eclipse.hyades.logging.events.cbe.util.EventFormatter;
import org.eclipse.hyades.logging.events.cbe.util.EventHelpers;

/**********************************************************************
 * Copyright (c) 2005 IBM Corporation and others.
 * All rights reserved.   This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * $Id: Logger.java,v 1.14 2005/04/19 02:51:51 paules Exp $
 * 
 * Contributors: 
 * IBM - Initial API and implementation
 **********************************************************************/

/**
 * Implementation of the Apache Commons
 * <code>org.apache.commons.logging.Log</code> interface which represents a
 * logger for logging messages to a Logging Agent with the same name as the
 * logger.
 * <p>
 * The logging level is used to filter which severity of logged messages are
 * sent to the Logging Agent with the same name as the logger. The logging level
 * may be set to one of the following logging levels:
 * <p>
 * -TRACE (least serious and most verbose) -DEBUG -INFO -WARN -ERROR -FATAL
 * (most serious and least verbose)
 * <p>
 * An instance of this class will be returned from the <code>getLog()</code>
 * and/or <code>getInstance()</code> APIs on the default
 * <code>org.apache.commons.logging.LogFactory</code> implementation. The
 * default <code>org.apache.commons.logging.LogFactory</code> implementation
 * uses the following discovery process to resolve the configured
 * <code>org.apache.commons.logging.Log</code> implementation:
 * <p>
 * 1) Look for a system property named
 * <code>org.apache.commons.logging.Log</code> or
 * <code>org.apache.commons.logging.log</code>. For example,
 * 
 * <code>...-Dorg.apache.commons.logging.Log=org.eclipse.hyades.logging.commons.Logger...</code>
 *  
 * - or -
 * 
 * <code>System.setProperty("org.apache.commons.logging.Log","org.eclipse.hyades.logging.commons.Logger");</code>
 * 
 * 2) Look for a configuration attribute of the default
 * <code>org.apache.commons.logging.LogFactory</code> implementation named
 * <code>org.apache.commons.logging.Log</code>. For example,
 * 
 * <code>org.apache.commons.logging.LogFactory.getFactory().setAttribute("org.apache.commons.logging.Log","org.eclipse.hyades.logging.commons.Logger");</code>
 * <p>
 * An instance of this class may be configured with a logging level using one of
 * the following methods (see above for the supported levels):
 * <p>
 * 1) Look for a system property named
 * <code>org.eclipse.hyades.logging.commons.Logger.LoggerLevel</code> at
 * logger creation. For example,
 * 
 * <code>...-Dorg.eclipse.hyades.logging.commons.Logger.Level=DEBUG...</code>
 *  
 * - or -
 * 
 * <code>System.setProperty("org.eclipse.hyades.logging.commons.Logger.Level","DEBUG");</code>
 * 
 * 2) Programmatically set the logging level. For example,
 * 
 * <code>logger.setLevelAsString("DEBUG");</code>
 *  
 * - or -
 * 
 * <code>logger.setLevel(Logger.DEBUG_LEVEL);</code>
 * <p>
 * A logger instance is created and returned when the <code>getLog()</code>
 * and/or <code>getInstance()</code> APIs on the default
 * <code>org.apache.commons.logging.LogFactory</code> implementation are
 * called. Once created, the logger is configurable (e.g. logging level). The
 * logger is generally consulted on its configuration before every message is
 * logged for performance reasons. For example,
 * <p>
 * <code>
 * System.setProperty("org.apache.commons.logging.Log", "org.eclipse.hyades.logging.commons.Logger");
 * System.setProperty("org.eclipse.hyades.logging.commons.Logger.Level", "DEBUG");
 * <p>
 * Log logger = LogFactory.getLog("My Logger");
 * <p>
 * if(logger.isDebugEnabled()){
 *     logger.debug("My first DEBUG log message.");
 *     logger.debug("My second DEBUG log message.");
 * }
 * </code>
 * <p>
 * Alternatively, users may use the proxy factory specific for <code>org.eclipse.hyades.logging.commons.Logger</code> 
 * instances (see <code>org.eclipse.hyades.logging.commons.LoggerFactory</code>).
 * <p> 
 * Logged <code>java.lang.Object</code> messages are converted to Common Base Event XML based on a 
 * generic serialization algorithm as implemented in <code>org.eclipse.hyades.logging.events.cbe.util.EventHelpers#convertObjectToCommonBaseEvent(Object,int)</code>
 * before being logged to the Logging Agent.
 * <p>
 * Callers may provide a Common Base Event XML file template (see 
 * org.eclipse.hyades.logging.core/schema/templateEvent.xsd) to configure the 
 * newly created Common Base Event before serialization to XML.  The template 
 * Event XML file is a well-formed XML document conforming to a predefined XML schema 
 * (e.g. templateEvent.xsd).  The naming convention used for the template Event XML file 
 * is:
 * <p>
 * &lt;logger name&gt;.event.xml
 * <p>
 * where &lt;logger name&gt; is the name of this logger.  For example, the template event XML file 
 * for the <code>myLogger</code> logger factory would be named <code>myLogger.event.xml</code>.
 * <p>
 * The desired event configuration template XML file must by on the classpath
 * and accessible by this class. The event configuration template XML file is
 * loaded using the following look-up sequence:
 * <p>
 * <ol>
 * <li>The <code>Logger</code> class' class loader
 * is queried for the event configuration template XML file.</li>
 * <li>The system's class loader is queried for the event configuration
 * template XML file.</li>
 * <li>The current thread's context class loader is queried for the event
 * configuration template XML file.</li>
 * </ol>
 * <p>
 * NOTE: The Apache Commons logging classes must be on the CLASSPATH at runtime
 * to utilize this logging class
 * <p>
 * 
 * 
 * @author Paul E. Slauenwhite
 * @version April 18, 2005
 * @since April 1, 2003
 * @see org.apache.commons.logging.Log
 * @see org.apache.commons.logging.LogFactory
 * @see org.eclipse.hyades.logging.core.LoggingAgent
 * @see org.eclipse.hyades.logging.commons.LoggerFactory
 */
public class Logger implements Log, IPreemptedLogger {

    /**
     * Trace logging level value.
     */
    public static final int TRACE_LEVEL = 0;

    /**
     * Debug logging level value.
     */
    public static final int DEBUG_LEVEL = 1;

    /**
     * Information logging level value.
     */
    public static final int INFO_LEVEL = 2;

    /**
     * Warning logging level value.
     */
    public static final int WARN_LEVEL = 3;

    /**
     * Error logging level value.
     */
    public static final int ERROR_LEVEL = 4;

    /**
     * Fatal logging level value.
     */
    public static final int FATAL_LEVEL = 5;

    /**
     * Array of String representations of the logging levels.
     */
    private static final String[] LEVEL_NAMES = { "TRACE", "DEBUG", "INFO", "WARN", "ERROR", "FATAL"};

    /**
     * The Logging Agent with the same name as the logger to which successfully
     * logged messages are sent.
     */
    protected LoggingAgent loggingAgent = null;

    /**
     * The logger's current logging level value.
     */
    private int level = -1;

    /**
     * The logger's name.
     */
    private String name = null;


    /**
     * Instance variable which holds a reference to a
     * <code>org.eclipse.hyades.logging.events.cbe.EventFactory</code> for
     * generating event instances.
     */
    private final static EventFactory EVENT_FACTORY = EventFactoryContext.getInstance().getSimpleEventFactoryHome().getAnonymousEventFactory();

    /**
     * Instance variable which holds a reference to a
     * <code>org.eclipse.hyades.logging.events.cbe.EventFactoryHome</code> for
     * generating pre-configured event instances.
     */
    private final static EventFactoryHome EVENT_FACTORY_HOME = EventFactoryContext.getInstance().getEventFactoryHome("org.eclipse.hyades.logging.events.cbe.impl.EventXMLFileEventFactoryHomeImpl", Logger.class.getClassLoader());

    /**
     * Constructor to create a logger instance with a Logging Agent using the
     * parameter name.
     * 
     * NOTE: The default logging level is set to WARN until explicitly set.
     * 
     * @param name
     *            The name of the newly created logger.
     */
    public Logger(String name) {

        this.level = WARN_LEVEL;
        this.name = name;

        loggingAgent = new LoggingAgent(name);

        configure();
    }

    /**
     * Checks if TRACE logging is currently enabled. TRACE logging infers the
     * logging level is currently set at TRACE or lower.
     * 
     * This API permits an inexpensive check on logging level.
     * 
     * @return True if TRACE logging is currently enabled, otherwise false.
     */
    public boolean isTraceEnabled() {
        return (level <= TRACE_LEVEL);
    }

    /**
     * Checks if DEBUG logging is currently enabled. DEBUG logging infers the
     * logging level is currently set at DEBUG or lower.
     * 
     * This API permits an inexpensive check on logging level.
     * 
     * @return True if DEBUG logging is currently enabled, otherwise false.
     */
    public boolean isDebugEnabled() {
        return (level <= DEBUG_LEVEL);
    }

    /**
     * Checks if INFO logging is currently enabled. INFO logging infers the
     * logging level is currently set at INFO or lower.
     * 
     * This API permits an inexpensive check on logging level.
     * 
     * @return True if INFO logging is currently enabled, otherwise false.
     */
    public boolean isInfoEnabled() {
        return (level <= INFO_LEVEL);
    }

    /**
     * Checks if WARN logging is currently enabled. WARN logging infers the
     * logging level is currently set at WARN or lower.
     * 
     * This API permits an inexpensive check on logging level.
     * 
     * @return True if WARN logging is currently enabled, otherwise false.
     */
    public boolean isWarnEnabled() {
        return (level <= WARN_LEVEL);
    }

    /**
     * Checks if ERROR logging is currently enabled. ERROR logging infers the
     * logging level is currently set at ERROR or lower.
     * 
     * This API permits an inexpensive check on logging level.
     * 
     * @return True if ERROR logging is currently enabled, otherwise false.
     */
    public boolean isErrorEnabled() {
        return (level <= ERROR_LEVEL);
    }

    /**
     * Checks if FATAL logging is currently enabled. FATAL logging infers the
     * logging level is currently set at FATAL or lower.
     * 
     * This API permits an inexpensive check on logging level.
     * 
     * @return True if FATAL logging is currently enabled, otherwise false.
     */
    public boolean isFatalEnabled() {
        return (level <= FATAL_LEVEL);
    }

    /**
     * Logs the parameter <code>java.lang.Object</code> log record to the
     * Logging Agent with the same name as the logger if TRACE logging is
     * currently enabled.
     * 
     * The log record is first converted to XML and then sent to the Logging
     * Agent.
     * 
     * @param record
     *            The log record to be logged to the Logging Agent.
     */
    public void trace(Object record) {

        if (isTraceEnabled()) loggingAgent.write(convertToXML(record));
    }

    /**
     * Logs the parameter <code>java.lang.Object</code> log record and
     * <code>java.lang.Throwable</code> exception to the Logging Agent with
     * the same name as the logger if TRACE logging is currently enabled.
     * 
     * The log record is first converted to XML and then sent to the Logging
     * Agent.
     * 
     * The parameter <code>java.lang.Object</code> log record is first logged
     * and then the <code>java.lang.Throwable</code> exception is logged to
     * the Logging Agent.
     * 
     * @param record
     *            The log record to be logged to the Logging Agent.
     * @param throwable
     *            The exception to be logged to the Logging Agent.
     */
    public void trace(Object record, Throwable throwable) {

        if (isTraceEnabled()) {
            loggingAgent.write(convertToXML(record));
            loggingAgent.write(convertToXML(throwable));
        }
    }

    /**
     * Logs the parameter <code>java.lang.Object</code> log record to the
     * Logging Agent with the same name as the logger if DEBUG logging is
     * currently enabled.
     * 
     * The log record is first converted to XML and then sent to the Logging
     * Agent.
     * 
     * @param record
     *            The log record to be logged to the Logging Agent.
     */
    public void debug(Object record) {

        if (isDebugEnabled()) loggingAgent.write(convertToXML(record));
    }

    /**
     * Logs the parameter <code>java.lang.Object</code> log record and
     * <code>java.lang.Throwable</code> exception to the Logging Agent with
     * the same name as the logger if DEBUG logging is currently enabled.
     * 
     * The log record is first converted to XML and then sent to the Logging
     * Agent.
     * 
     * The parameter <code>java.lang.Object</code> log record is first logged
     * and then the <code>java.lang.Throwable</code> exception is logged to
     * the Logging Agent.
     * 
     * @param record
     *            The log record to be logged to the Logging Agent.
     * @param throwable
     *            The exception to be logged to the Logging Agent.
     */
    public void debug(Object record, Throwable throwable) {

        if (isDebugEnabled()) {
            loggingAgent.write(convertToXML(record));
            loggingAgent.write(convertToXML(throwable));
        }
    }

    /**
     * Logs the parameter <code>java.lang.Object</code> log record to the
     * Logging Agent with the same name as the logger if INFO logging is
     * currently enabled.
     * 
     * The log record is first converted to XML and then sent to the Logging
     * Agent.
     * 
     * @param record
     *            The log record to be logged to the Logging Agent.
     */
    public void info(Object record) {

        if (isInfoEnabled()) loggingAgent.write(convertToXML(record));
    }

    /**
     * Logs the parameter <code>java.lang.Object</code> log record and
     * <code>java.lang.Throwable</code> exception to the Logging Agent with
     * the same name as the logger if INFO logging is currently enabled.
     * 
     * The log record is first converted to XML and then sent to the Logging
     * Agent.
     * 
     * The parameter <code>java.lang.Object</code> log record is first logged
     * and then the <code>java.lang.Throwable</code> exception is logged to
     * the Logging Agent.
     * 
     * @param record
     *            The log record to be logged to the Logging Agent.
     * @param throwable
     *            The exception to be logged to the Logging Agent.
     */
    public void info(Object record, Throwable throwable) {

        if (isInfoEnabled()) {
            loggingAgent.write(convertToXML(record));
            loggingAgent.write(convertToXML(throwable));
        }
    }

    /**
     * Logs the parameter <code>java.lang.Object</code> log record to the
     * Logging Agent with the same name as the logger if WARN logging is
     * currently enabled.
     * 
     * The log record is first converted to XML and then sent to the Logging
     * Agent.
     * 
     * @param record
     *            The log record to be logged to the Logging Agent.
     */
    public void warn(Object record) {

        if (isWarnEnabled()) loggingAgent.write(convertToXML(record));
    }

    /**
     * Logs the parameter <code>java.lang.Object</code> log record and
     * <code>java.lang.Throwable</code> exception to the Logging Agent with
     * the same name as the logger if WARN logging is currently enabled.
     * 
     * The log record is first converted to XML and then sent to the Logging
     * Agent.
     * 
     * The parameter <code>java.lang.Object</code> log record is first logged
     * and then the <code>java.lang.Throwable</code> exception is logged to
     * the Logging Agent.
     * 
     * @param record
     *            The log record to be logged to the Logging Agent.
     * @param throwable
     *            The exception to be logged to the Logging Agent.
     */
    public void warn(Object record, Throwable throwable) {

        if (isWarnEnabled()) {
            loggingAgent.write(convertToXML(record));
            loggingAgent.write(convertToXML(throwable));
        }
    }

    /**
     * Logs the parameter <code>java.lang.Object</code> log record to the
     * Logging Agent with the same name as the logger if ERROR logging is
     * currently enabled.
     * 
     * The log record is first converted to XML and then sent to the Logging
     * Agent.
     * 
     * @param record
     *            The log record to be logged to the Logging Agent.
     */
    public void error(Object record) {

        if (isErrorEnabled()) loggingAgent.write(convertToXML(record));
    }

    /**
     * Logs the parameter <code>java.lang.Object</code> log record and
     * <code>java.lang.Throwable</code> exception to the Logging Agent with
     * the same name as the logger if ERROR logging is currently enabled.
     * 
     * The log record is first converted to XML and then sent to the Logging
     * Agent.
     * 
     * The parameter <code>java.lang.Object</code> log record is first logged
     * and then the <code>java.lang.Throwable</code> exception is logged to
     * the Logging Agent.
     * 
     * @param record
     *            The log record to be logged to the Logging Agent.
     * @param throwable
     *            The exception to be logged to the Logging Agent.
     */
    public void error(Object record, Throwable throwable) {

        if (isErrorEnabled()) {
            loggingAgent.write(convertToXML(record));
            loggingAgent.write(convertToXML(throwable));
        }
    }

    /**
     * Logs the parameter <code>java.lang.Object</code> log record to the
     * Logging Agent with the same name as the logger if FATAL logging is
     * currently enabled.
     * 
     * The log record is first converted to XML and then sent to the Logging
     * Agent.
     * 
     * @param record
     *            The log record to be logged to the Logging Agent.
     */
    public void fatal(Object record) {

        if (isFatalEnabled()) loggingAgent.write(convertToXML(record));
    }

    /**
     * Logs the parameter <code>java.lang.Object</code> log record and
     * <code>java.lang.Throwable</code> exception to the Logging Agent with
     * the same name as the logger if FATAL logging is currently enabled.
     * 
     * The log record is first converted to XML and then sent to the Logging
     * Agent.
     * 
     * The parameter <code>java.lang.Object</code> log record is first logged
     * and then the <code>java.lang.Throwable</code> exception is logged to
     * the Logging Agent.
     * 
     * @param record
     *            The log record to be logged to the Logging Agent.
     * @param throwable
     *            The exception to be logged to the Logging Agent.
     */
    public void fatal(Object record, Throwable throwable) {

        if (isFatalEnabled()) {
            loggingAgent.write(convertToXML(record));
            loggingAgent.write(convertToXML(throwable));
        }
    }

    /**
     * Retrieves the name of this logger.
     * 
     * @return Name of this logger.
     */
    public String getName() {
        return name;
    }

    /**
     * Retrieves the integer level of this logger.
     * 
     * @return Integer logging level of this logger (i.e. TRACE_LEVEL -
     *         FATAL_LEVEL).
     */
    public int getLevel() {
        return level;
    }

    /**
     * Retrieves the string level of this logger.
     * 
     * @return String logging level of this logger (i.e. "TRACE" - "FATAL").
     */
    public String getLevelAsString() {
        return LEVEL_NAMES[level];
    }

    /**
     * Sets the logging level to one of the integer logging levels (i.e.
     * TRACE_LEVEL - FATAL_LEVEL).
     * 
     * @param level
     *            The new integer logging level (i.e. TRACE_LEVEL -
     *            FATAL_LEVEL).
     * @exception java.lang.IllegalArgumentException
     *                Attempting to set an illegal integer logging level.
     */
    public synchronized void setLevel(int level) throws IllegalArgumentException {

        if (isValidLevel(level)) {
            this.level = level;
        } else {
            throw new IllegalArgumentException("Invalid logging level");
        }
    }

    /**
     * Sets the logging level to one of the string logging levels (i.e. "TRACE" -
     * "FATAL").
     * 
     * @param stringLevel
     *            The new string logging level (i.e. "TRACE" - "FATAL").
     * @exception java.lang.IllegalArgumentException
     *                Attempting to set an illegal string logging level.
     */
    public synchronized void setLevelAsString(String stringLevel) throws IllegalArgumentException {
        setLevel(getLevelInteger(stringLevel));
    }

    /**
     * Returns the integer level value for the parameter level String.
     * 
     * The parameter level string could be either the English (i.e. "TRACE" -
     * "FATAL") or the level number as a string (i.e. TRACE_LEVEL -
     * FATAL_LEVEL).
     * 
     * @param levelName
     *            a potential level string to be checked for its integer value.
     * @return The integer level value for the parameter level String, or -1 if
     *         invalid.
     */
    private int getLevelInteger(String levelName) {

        if ((levelName != null) && (levelName.trim().length() != 0)) {

            String tempLevel = levelName.trim();

            //Check against the English levels:
            for (int counter = 0; counter < LEVEL_NAMES.length; counter++) {

                if (tempLevel.equalsIgnoreCase(LEVEL_NAMES[counter])) { return counter; }
            }

            //Check for a valid integer:
            try {
                int levelNum = Integer.parseInt(tempLevel);

                if (isValidLevel(levelNum)) { return levelNum; }
            } catch (NumberFormatException n) {
            }
        }

        return -1;
    }

    /**
     * Checks if the parameter check level is one of the valid integer logging
     * levels (i.e. TRACE_LEVEL - FATAL_LEVEL).
     * 
     * @param checkLevel
     *            a potential level to be checked for validity.
     * @return true if the parameter check level is one of the valid integer
     *         logging levels (i.e. TRACE_LEVEL - FATAL_LEVEL), otherwise false.
     */
    private boolean isValidLevel(int checkLevel) {
        return ((checkLevel >= TRACE_LEVEL) || (checkLevel <= FATAL_LEVEL));
    }

    /**
     * Checks if the parameter check level name is one of the valid string
     * logging level (i.e. "TRACE" - "FATAL").
     * 
     * @param checkLevel
     *            a potential level name to be checked for validity.
     * @return true if the parameter check level name is one of the valid string
     *         logging level (i.e. "TRACE" - "FATAL"), otherwise false.
     */
    private boolean isValidLevelString(String checkLevel) {
        return (isValidLevel(getLevelInteger(checkLevel)));
    }

    /**
     * Internal utility API for converting the parameter
     * <code>java.lang.Object</code> to Common Base Event 
     * XML based on a generic serialization algorithm as 
	 * implemented in <code>org.eclipse.hyades.logging.events.cbe.util.EventHelpers#convertObjectToCommonBaseEvent(Object,int)</code>.
	 * <p>
     * Callers may provide a Common Base Event XML file template (see 
     * org.eclipse.hyades.logging.core/schema/templateEvent.xsd) to configure the 
     * newly created Common Base Event before serialization to XML.  See class comment 
     * header for more details.
     * <p>
     * 
     * @param object
     *            The object to be converted to XML.
     * @return The XML fragment of the converted parameter object.
     */
    private String convertToXML(Object object) {

        if (object instanceof IExternalizableToXml) {
            return ((IExternalizableToXml) object).externalizeCanonicalXmlString();
        } 
            
        //Map the parameter Object's properties to Common Base Event properties:
        //Retrieve an EventFactory instance based on the logger name and create a pre-configured Common Base Event based 
        //on the an XML configuration template file that exists on the class loader's or context class loader's classpath.
        CommonBaseEvent commonBaseEvent = EVENT_FACTORY_HOME.getEventFactory(name).createCommonBaseEvent();
		    
        try {
        	commonBaseEvent.complete();
        } 
        catch (CompletionException c) {
        	//Ignore since content completion is based on 'best-effort'.
        }

        EventHelpers.convertObjectToCommonBaseEvent(commonBaseEvent, object,4);
        
        commonBaseEvent.addExtendedDataElement("Logger_Name", name);
        commonBaseEvent.addExtendedDataElement("Logging_Level", LEVEL_NAMES[level]);
        
        return (EventFormatter.toCanonicalXMLString(commonBaseEvent, true).concat(Constants.LINE_SEPARATOR));
    }

    /**
     * De-registers this Logging Agent instance from the Agent Controller.
     * 
     * @see java.lang.Object#finalize()
     */
    public final void finalize() {
        loggingAgent.deregister();
    }

    /**
     * Suspends the current thread until the Logging Agent is monitored or the
     * the parameter maximum time (milliseconds) has expired.
     * 
     * @param maxTime
     *            the maximum amount of time (milliseconds) to suspend the
     *            current thread
     * @return true if the logger is being monitored
     */
    public boolean waitUntilLogging(long maxTime) {
        return (loggingAgent.waitUntilLogging(maxTime));
    }

    /**
     * Checks if the Logging Agent is logging (e.g. Agent Controller is
     * available and the Logging Agent is being monitored).
     * 
     * @return True if the Logging Agent is logging, otherwise false.
     */
    public boolean isLogging() {
        return (loggingAgent.isLogging());
    }

    /**
     * Configures the logger using system properties.
     */
    private void configure() {

        String systemProperty = System.getProperty("org.eclipse.hyades.logging.commons.Logger.Level");

        if ((systemProperty != null) && (systemProperty.trim().length() > 0) && (isValidLevelString(systemProperty))) {
            setLevelAsString(systemProperty);
        }
    }

    /**
     * Test main method.
     * 
     * @param args
     *            public static void main(String[] args) {
     * 
     * System.setProperty("org.apache.commons.logging.Log",
     * "org.eclipse.hyades.logging.commons.Logger");
     * System.setProperty("org.eclipse.hyades.logging.commons.Logger.Level",
     * "DEBUG");
     * 
     * Log logger = LogFactory.getLog("My Logger");
     * 
     * if (logger.isDebugEnabled()) { logger.debug("My first DEBUG log
     * message."); logger.debug("My second DEBUG log message."); }
     * 
     * System.out.println("Done!"); }
     */
}