/*******************************************************************************
 * Copyright (c) 2005 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * $Id: DeploymentLaunchConfigurationFacade.java,v 1.7 2005/02/16 22:22:13 qiyanli Exp $
 * 
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
package org.eclipse.hyades.test.ui.launch.configurations;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.debug.core.ILaunchConfiguration;
import org.eclipse.debug.core.ILaunchConfigurationWorkingCopy;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.hyades.models.common.configuration.util.ConfigurationUtil;
import org.eclipse.hyades.models.common.testprofile.TPFDeployment;
import org.eclipse.hyades.models.common.testprofile.TPFTest;
import org.eclipse.hyades.models.common.testprofile.TPFTestCase;
import org.eclipse.hyades.models.common.testprofile.TPFTestSuite;
import org.eclipse.hyades.test.ui.internal.model.EMFUtil;
import org.eclipse.hyades.test.ui.util.DeploymentUtil;

/**
 * This class is a facade for accessing Deployment information in a Hyades Launch Configuration.
 * @author jcanches
 */
public class DeploymentLaunchConfigurationFacade extends AbstractLaunchConfigurationFacade {

	private static final String DEFAULT_DEPLOYMENT_VALUE = "default";
	private final static String PROPERTY_DEPLOYMENT = PROPERTY_PREFIX + ".deployment"; //$NON-NLS-1$

	/**
	 * Retrieves the Deployment property of a Hyades Launch Configuration. This property defines
	 * which Deployment will be used to run the Launch Configuration's test.
	 * @deprecated
	 * @see DeploymentLaunchConfigurationFacade#getDeployment(ILaunchConfiguration, ResourceSet)
	 */
	public synchronized static TPFDeployment getDeployment(ILaunchConfiguration configuration) throws CoreException {
		return getDeployment(configuration, EMFUtil.getResourceSet());
	}
	
	/**
	 * Retrieves the Deployment property of a Hyades Launch Configuration. This property defines
	 * which Deployment will be used to run the Launch Configuration's test.
	 * @param configuration
	 * @param resourceSet The Resource Set that should contain the returned Deployment.
	 * @since 3.1
	 */
	public synchronized static TPFDeployment getDeployment(ILaunchConfiguration configuration, ResourceSet resourceSet) throws CoreException {
		TPFDeployment deployment = null;
		if (DEFAULT_DEPLOYMENT_VALUE.equals(configuration.getAttribute(PROPERTY_DEPLOYMENT, (String)null))) {
			// This is the default deployment
			deployment = ConfigurationUtil.createDefaultDeployment();
		} else {
			EObject eObject = resolveEMFReference(configuration, PROPERTY_DEPLOYMENT, resourceSet);
			if (eObject != null && eObject instanceof TPFDeployment) {
				deployment = (TPFDeployment)eObject;
			}
		}
		
		if (deployment != null) {
			// added to make sure deployment contains a valid pair of location and artifact for this test suite
			// and automatically calculate CLASSPATH for this test
			// bugzilla# 58827. bjiang 4/21/2004
			TPFTest test = TestLaunchConfigurationFacade.getTest(configuration, resourceSet);
			TPFTestSuite testSuite = null;
			if (test instanceof TPFTestSuite) {
				testSuite = (TPFTestSuite)test;
			} else if (test instanceof TPFTestCase) {
				testSuite = ((TPFTestCase)test).getTestSuite();
			}
			
			if(testSuite != null) {
				DeploymentUtil.associateTestAsset(testSuite, deployment);
			}
		}
		return deployment;
	}
	
	/**
	 * Define the Deployment property of a Hyades Launch Configuration. This property defines
	 * which Deployment will be used to run the Launch Configuration's test.
	 * @param configuration The Launch Configuration Working Copy that receives the deployment property.
	 * @param The deployment to be associated with the Launch Configuration. The deployment must
	 * be either the default deployment, or a deployment that is contained by an EMF Resource.
	 */	
	public synchronized static void setDeployment(ILaunchConfigurationWorkingCopy configuration, TPFDeployment deployment) {
		if (ConfigurationUtil.isDefaultDeployment(deployment)) {
			// Remove any previous EMF Reference associated to the launch configuration
			unsetEMFReference(configuration, PROPERTY_DEPLOYMENT);
			// Set the deployment property to "default"
			configuration.setAttribute(PROPERTY_DEPLOYMENT, DEFAULT_DEPLOYMENT_VALUE);
		} else {
			// Remove the "default deployment" attribute
			configuration.setAttribute(PROPERTY_DEPLOYMENT, (String)null);
			// Store the EMF reference to the deployment
			storeEMFReference(configuration, PROPERTY_DEPLOYMENT, deployment);
		}
	}
	
}
